#!/usr/bin/python

import os
import xml.sax
from xml.sax.saxutils import quoteattr,escape
import sys
import codecs
import argparse
import re
import pprint

import xml.sax.saxutils

# This program is used for updating an evergreen authorities database with authorities from an up to date
# file, in this case from NK. The program parses the EG authorities export, finds their universal
# IDs and saves them in memory. It then parses the NK authorities and tries to match their IDs with
# those of EG. If a match is found, the NK authority is printed out to the output file with the
# appropriate EG ID in the form of: "EG_ID	XML_data_of_NK". This can be then used to update the EG
# database.


# SAX parser of EG authorities file
class egExportParser(xml.sax.handler.ContentHandler):
	def startDocument(self):
		self.datafield=None
		self.subfield=None
		self.controlfield=None
		self.egSubfield=None        
		self.nkSubfield=None        
		self.counter=0
		self.wrong_datafield=True

	def startElement(self,tag,attrs):
		
		if tag=="datafield":
			self.datafield=attrs.get('tag')
			self.subfield=None
		elif tag=="subfield":
			self.subfield=attrs.get('code')
			self.egSubfield=None
		elif tag=="record": 

			self.wrong_datafield=True

			self.datafield=None
			self.subfield=None
			self.counter+=1
			if not self.counter%1000:
				print ".",
				sys.stdout.flush()

	def characters(self,data):
		data = data.strip()
		if self.datafield=="035" and self.subfield=="a":
			if "(CZ PrNK)" in data:
				data = data.replace("(CZ PrNK)", "")

			if data!="":
				self.nkSubfield = data

		if self.datafield=="040":
			self.wrong_datafield=False

		if self.datafield=="901" and self.subfield=="c":
			if data!="":
				self.egSubfield=data

		if self.egSubfield and self.nkSubfield:
			nkIdsFromEg[self.nkSubfield]=self.egSubfield

			self.egSubfield=None
			self.nkSubfield=None
			
	def endElement(self, tag):
		if tag=="record" and self.wrong_datafield:
			print "FUUUCK\n"

# SAX parser of the up to date NK file
class nkExportParser(xml.sax.handler.ContentHandler):
	def startDocument(self):
		self.controlfield=None
		self.datafield=None
		self.subfield=None
		self.record=None
		self.recordData=None
		self.isRecord=False
		self.counter=0
		self.txtXml=""
		self.idIn=None
		self.writeToOutput=False
		self.writeToCut=False
		self.ind1=None
		self.ind2=None

	def startElement(self,tag,attrs):
		if tag=="datafield":
			self.subfield=None
			self.controlfield=None

			self.datafield=attrs.get('tag')
			self.ind1=attrs.get('ind1')
			self.ind2=attrs.get('ind2')

			self.subfield=None
			if self.datafield!=None and self.ind1!=None and self.ind2!=None:
				self.txtXml+='<datafield tag=\"'+self.datafield+'\"'
				self.txtXml+=' ind1=\"'+self.ind1+'\" ind2=\"'+self.ind2+'\">'
			if self.datafield!=None and self.ind1!=None and self.ind2==None:
				self.txtXml+='<datafield tag=\"'+self.datafield+'\"'
				self.txtXml+=' ind1=\"'+self.ind1+'\">'
			if self.datafield!=None and self.ind1==None and self.ind2!=None:
				self.txtXml+='<datafield tag=\"'+self.datafield+'\"'
				self.txtXml+=' ind2=\"'+self.ind2+'\">'
			elif self.datafield==None:
				self.txtXml+='<datafield>'

		elif tag=="subfield":
			self.datafield=None
			self.controlfield=None

			self.subfield=attrs.get('code')
			self.egSubfield=None
			if self.subfield!=None:
				self.txtXml+='<subfield code=\"'+self.subfield+'\">'
			elif self.subfield==None:
				self.txtXml+='<subfield>'

		elif tag=="controlfield":
			self.datafield=None
			self.subfield=None

			self.controlfield=attrs.get('tag')
			if self.controlfield!=None:
				self.txtXml+='<controlfield tag=\"'+self.controlfield+'\">'
			elif self.controlfield==None:
				self.txtXml+='<controlfield>'

		elif tag=="record": 
			self.datafield=None
			self.subfield=None
			self.controlfield=None
			self.counter+=1
			if not self.counter%1000:
				print ".",
				sys.stdout.flush()
			self.txtXml+='<record>\n'

		elif tag=="leader":
			self.txtXml+='<leader>'

	def endElement(self,tag):
		if tag=="datafield":
			self.txtXml+='</datafield>\n'
			self.ind1=None
			self.ind2=None
		elif tag=="subfield":
			self.txtXml+='</subfield>\n'
		elif tag=="controlfield":
			self.txtXml+='</controlfield>\n'
		elif tag=="leader":
			self.txtXml+='</leader>\n'
		elif tag=="record": 
			self.txtXml+='</record>\n'

			if self.writeToOutput:
				self.txtXml=self.txtXml.replace('\n', '')
				outXmlFile.write(self.txtXml)
				outXmlFile.write("\n")
			elif self.writeToCut:
				cutXmlFile.write(self.txtXml)				
			self.writeToOutput=False
			self.txtXml=""
			
	def characters(self,data):
		dataEscaped = xml.sax.saxutils.escape(data)
		self.txtXml += dataEscaped.rstrip()
		if self.controlfield=="001":
			self.idIn = nkIdsFromEg.get(data)
			if self.idIn!=None:
				
				outXmlFile.write(nkIdsFromEg[data] + "	")
				
				del nkIdsFromEg[data]
				self.writeToOutput=True
				self.writeToCut=True


# list of universal IDs parsed from the EG file
nkIdsFromEg = {}

# parser that interacts with the user
parser = argparse.ArgumentParser(description='Hello there')

# paths to the individual files to parse
parser.add_argument("egFile",nargs='?',help="export XML file",default="../autority_zaklad/EG_auth_UTF-8.xml") 
parser.add_argument("nkFile",nargs='?',help="authority XML file",default="cut0.xml")
parser.add_argument("-o","--output",help="output file",default="akt_output.txt")
parser.add_argument("-c","--cut_file",help="cut file, without records used in previous script",default="cut.xml")
parser.add_argument("-d","--deleted",help="file of deleted records",default="deleted.txt")

args = parser.parse_args()

egFile=args.egFile
nkFile=args.nkFile
pathOut = args.output
cut_file = args.cut_file
deleted_file = args.deleted

# writing the correct XML header to the begining of the created XML files
outXmlFile = codecs.open(pathOut,"w","utf-8")
cutXmlFile = codecs.open(cutFile,"w","utf-8")
cutXmlFile.write("""<?xml version = "1.0" encoding = "UTF-8"?>\n""")
cutXmlFile.write("""<collection xmlns="http://www.loc.gov/MARC21/slim" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://www.loc.gov/MARC21/slim http://www.loc.gov/standards/marcxml/schema/MARC21slim.xsd">\n""")

egExportParser = egExportParser()
parser = xml.sax.parse(egFile, egExportParser)

# print out the number of records we went through
#pprint.pprint(nkIdsFromEg)
print("Projetych zaznamu z EG: ", len(nkIdsFromEg))

nkExportParser = nkExportParser()
parser = xml.sax.parse(nkFile, nkExportParser)

outXmlFile.close()
cutXmlFile.write("</collection>")
cutXmlFile.close()

# output the deleted files
with open(deleted_file, 'wt') as out:
	pprint.pprint(nkIdsFromEg, stream=out)

print "Konec :]"