#!/usr/bin/python

import os
import xml.sax
from xml.sax.saxutils import quoteattr,escape
import sys
import codecs
import argparse
import re

# This program adds authorities from an up to date file, here NK export file, to EG database based on matching
# bibliographic records from EG database and a cut file of authorities from NK. The cut file is the original
# NK export without the records, which have already been matched by the updating script a.py.

# SAX parser of EG bibliographic file
class egExportParser(xml.sax.handler.ContentHandler):
	def startDocument(self):
		self.datafield=None
		self.subfield=None
		self.controlfield=None
		self.egSubfield=None        
		self.nkSubfield=None        
		self.counter=0
		self.wrongData=False

		# the file pole.txt contains list of IDs of datafields, that are being compared for a match
		self.setOfDatafields=words =  set(open(pole).read().split())

	def startElement(self,tag,attrs):
		if tag=="datafield":
			self.datafield=attrs.get('tag')
			self.subfield=None
		elif tag=="subfield":
			self.subfield=attrs.get('code')
			self.egSubfield=None
		elif tag=="record": 
			self.datafield=None
			self.subfield=None
			self.counter+=1
			if not self.counter%1000:
				print ".",
				sys.stdout.flush()

	def characters(self,data):
		data = data.strip()
	
		if (self.datafield in self.setOfDatafields) and (self.subfield=="7"):
			if data!="":
				self.nkSubfield = data

		if self.nkSubfield:
			nkIdsFromEg.add(self.nkSubfield)

			self.nkSubfield=None

# SAX parser of the cut NK file
class nkExportParser(xml.sax.handler.ContentHandler):
	def startDocument(self):
		self.controlfield=None
		self.datafield=None
		self.subfield=None
		self.record=None
		self.recordData=None
		self.isRecord=False
		self.counter=0
		self.txtXml=""
		self.idIn=None
		self.writeToOutput=False
		self.writeToCut=False
		self.ind1=None
		self.ind2=None

	def startElement(self,tag,attrs):
		if tag=="datafield":
			self.datafield=attrs.get('tag')
			self.ind1=attrs.get('ind1')
			self.ind2=attrs.get('ind2')

			self.subfield=None
			if self.datafield!=None and self.ind1!=None and self.ind2!=None:
				self.txtXml+='<datafield tag=\"'+self.datafield+'\"'
				self.txtXml+=' ind1=\"'+self.ind1+'\" ind2=\"'+self.ind2+'\">'
			if self.datafield!=None and self.ind1!=None and self.ind2==None:
				self.txtXml+='<datafield tag=\"'+self.datafield+'\"'
				self.txtXml+=' ind1=\"'+self.ind1+'\">'
			if self.datafield!=None and self.ind1==None and self.ind2!=None:
				self.txtXml+='<datafield tag=\"'+self.datafield+'\"'
				self.txtXml+=' ind2=\"'+self.ind2+'\">'
			elif self.datafield==None:
				self.txtXml+='<datafield>'

		elif tag=="subfield":
			self.subfield=attrs.get('code')
			self.egSubfield=None
			if self.subfield!=None:
				self.txtXml+='<subfield code=\"'+self.subfield+'\">'
			elif self.subfield==None:
				self.txtXml+='<subfield>'

		elif tag=="controlfield":
			self.controlfield=attrs.get('tag')
			if self.controlfield!=None:
				self.txtXml+='<controlfield tag=\"'+self.controlfield+'\">'
			elif self.controlfield==None:
				self.txtXml+='<controlfield>'

		elif tag=="record": 
			self.datafield=None
			self.subfield=None
			self.counter+=1
			if not self.counter%1000:
				print ".",
				sys.stdout.flush()
			self.txtXml+='<record>'

		elif tag=="leader":
			self.txtXml+='<leader>'

	def endElement(self,tag):
		if tag=="datafield":
			self.txtXml+='</datafield>\n'
		elif tag=="subfield":
			self.txtXml+='</subfield>\n'
		elif tag=="controlfield":
			self.txtXml+='</controlfield>\n'
		elif tag=="leader":
			self.txtXml+='</leader>\n'
		elif tag=="record": 
			self.txtXml+='</record> \n'

			if self.writeToOutput:
				self.txtXml=self.txtXml
				outXmlFile.write(self.txtXml)
			elif self.writeToCut:
				cutXmlFile.write(self.txtXml)

			self.writeToOutput=False
			self.txtXml=""
			
	def characters(self,data):
		dataEscaped = xml.sax.saxutils.escape(data)
		self.txtXml += dataEscaped.rstrip()

		if self.controlfield=="001":
			if data in nkIdsFromEg:
				self.writeToOutput=True
				self.writeToCut=True


# list of IDs of bibliographic records from EG
nkIdsFromEg = set()

# parser that interacts with the user
parser = argparse.ArgumentParser(description='Nazdarek')

# paths to the individual files to parse
parser.add_argument("egFile",nargs='?',help="export XML file",default="../autority_zaklad/EG_auth_UTF-8.xml") 
parser.add_argument("nkFile",nargs='?',help="authority XML file",default="cut.xml")
parser.add_argument("-o","--output",help="output file",default="bib_output.xml")
parser.add_argument("-c","--cutFile",help="cut file",default="cut2.xml")
parser.add_argument("-p","--pole",help="list of IDs which are being compared",default="pole.txt")

args = parser.parse_args()

egFile=args.egFile
nkFile=args.nkFile
pathOut = args.output
cutFile = args.cutFile
pole = args.pole

outXmlFile = codecs.open(pathOut,"w","utf-8")
outXmlFile.write("""<?xml version = "1.0" encoding = "UTF-8"?>\n""")
outXmlFile.write("""<collection>\n""")

# writing the correct XML header to the begining of the created XML files
cutXmlFile = codecs.open(cutFile,"w","utf-8")
cutXmlFile.write("""<?xml version = "1.0" encoding = "UTF-8"?>\n""")
cutXmlFile.write("""<collection xmlns="http://www.loc.gov/MARC21/slim" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://www.loc.gov/MARC21/slim http://www.loc.gov/standards/marcxml/schema/MARC21slim.xsd">\n""")

egExportParser = egExportParser()
parser = xml.sax.parse(egFile, egExportParser)

# print out the number of records we went through
print("Projetych zaznamu z EG: ", len(nkIdsFromEg))

nkExportParser = nkExportParser()
parser = xml.sax.parse(nkFile, nkExportParser)
cutXmlFile.write("</collection>")
outXmlFile.write("</collection>")

cutXmlFile.close()
outXmlFile.close()
print "Konec :]"