#!/usr/bin/python

import os
import xml.sax
from xml.sax.saxutils import quoteattr,escape
import sys
import codecs
import argparse
import re

class ExportParser(xml.sax.handler.ContentHandler):
    def startDocument(self):
        self.datafield=None
        self.subfield=None        
        self.isRecordIDfield=False
        self.counter=0

    def endDocument(self):
        print "\nProcessing finished\n\n"
        
    def startElement(self,tag,attrs):        
        if tag=="datafield":
            self.datafield=attrs.get('tag')
            self.subfield=None          
        elif tag=="subfield":
            self.subfield=attrs.get('code')
        elif tag=="record":
            self.datafield=None
            self.subfield=None
            self.counter+=1
            if not self.counter%1000:
                print ".",
                sys.stdout.flush()

    def endElement(self,tag):
        self.isRecordIDfield=False

    def characters(self,data):
        if self.isRecordIDfield:
            self.recordId=data.strip()
        else:
            code = "%s%s"%(self.datafield,self.subfield)
            if code in exportedFields:
                data = data.strip()
                if data:
                    authorsExported.add(get_value(data))
        
class AuthorityParser(xml.sax.handler.ContentHandler):
    def startDocument(self):
        self.recordTags=[]
        self.includeRecord=False
        self.counter=0

    def endDocument(self):
        print "\nProcessing finished\n\n"

        
    def startElement(self,tag,attrs):                        
        aaa = ""
        for nm,val in attrs.items():
            aaa+=" %s=%s"%(nm,quoteattr(val))
        self.recordTags.append("\n<%s%s>"%(tag,aaa))
        
        if tag=="datafield":
            self.datafield=attrs.get('tag')
            self.subfield=None          
        elif tag=="subfield":
            self.subfield=attrs.get('code')
        elif tag=="record":
            self.datafield=None
            self.subfield=None
            self.counter+=1
            if not self.counter%1000:
                print ".",
                sys.stdout.flush()
            
    def endElement(self,tag):
        self.recordTags.append("</%s>"%tag)
        if tag=="record":
            if self.includeRecord:
                xxx.write("".join(self.recordTags).replace('></','>\n</'))
            self.recordTags=[]
            self.includeRecord=False

    def characters(self,data):
        if data.strip():            
            self.recordTags.append(escape(data))
            code = "%s%s"%(self.datafield,self.subfield)
            if code in authorityFields and get_value(data) in authorsExported:
                self.includeRecord=True


parser = argparse.ArgumentParser(description='Extract records for existing authors from authority xml file')

parser.add_argument("exportFile",nargs='?',help="export XML file",default="exported_files_UTF8.xml") 
parser.add_argument("authorityFile",nargs='?',help="authority XML file",default="aut_exp.xml")

parser.add_argument("mappingFile",nargs='?',help="mapping file",default="bibout.txt")

parser.add_argument("-o","--output",help="output file",default="/tmp/biblio_output.xml")

parser.add_argument("-m","--match",help="string match (number: length, anything else: first separator. )",default="__ALL__")

parser.add_argument("-f","--filter_chars",help="prefilter string; e.g. \",;\"",default=";,")


args = parser.parse_args()

exportFile=args.exportFile
authorityFile=args.authorityFile
mappingFile=args.mappingFile
pthOut = args.output


match = args.match
try:
    match=int(match)
except:
    if match=="__ALL__":
        match=0

print "Use value:",
if match:
    if isinstance(match,int):
        print "length %s characters"%match
    else:
        print "before character: %s"%match
else:
    print "everything"

filter_chars = args.filter_chars
    
def get_value(txt):
    if filter_chars:
        txt = re.compile("[%s]"%filter_chars).sub('',txt)
    
    if match:
        if isinstance(match,int):
            return txt[:match]
        else:
            return txt.partition(match)[0].strip()
    else:
        return txt


xxx = codecs.open(pthOut,"w","utf-8")
xxx.write('<collection xmlns="http://www.loc.gov/MARC21/slim" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.loc.gov/MARC21/slim http://www.loc.gov/standards/marcxml/schema/MARC21slim.xsd">')


dataDir="/tmpPersistent/JabokXML/"

mapDct = dict()
exportedFields = set()
authorityFields = set()

authorsExported = set()


def parse_mapping_file():
    print "Reading mapping file: %s"%(mappingFile)
    xxx = open(mappingFile)
    for ln in xxx:
        f,t = [x.strip() for x in ln.split(':')]
        mapDct.setdefault(f,[]).append(t)
        exportedFields.add(f)
        authorityFields.add(t)
parse_mapping_file()


print "Processing  export file: %s"%(exportFile)
pExported = ExportParser()
parser = xml.sax.parse(exportFile,pExported)
print len(authorsExported)


print "Processing authority file: %s"%(authorityFile)
aExported = AuthorityParser()
parser = xml.sax.parse(authorityFile,aExported)
xxx.write('</collection>')
xxx.close()

print "Output file: ",pthOut
